//---------------------------------------------------------------------------
// Version: 1.0.0
// Date: 2012-04-08 UTC+09:00
// Author: Moochos
//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop
#include "Moa_AutoPointers.h"
#include "Moa_ShellLibrary.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
//---------------------------------------------------------------------------
namespace Moa_shelllibrary {
//---------------------------------------------------------------------------
void __fastcall TDroppedFiles::AssureUnfinished()
{
	if(IsFinished()) {
		throw EFileDropError("The file drop operation is already finished.");
	}
}
//---------------------------------------------------------------------------
AnsiString __fastcall TDroppedFiles::GetFile(int AIndex)
{
	AssureUnfinished();
	if(AIndex < 0 || AIndex >= Count) {
		throw EFileDropError("The program cannot get the path of a dropped file.");
	}
	UINT LBufferSize = ::DragQueryFile(Handle, AIndex, NULL, 0) + 1;
	TAutoArray<TCHAR> LBuffer = new TCHAR[LBufferSize];
	UINT LPathLength = ::DragQueryFile(Handle, AIndex, LBuffer, LBufferSize);
	return AnsiString(LBuffer, LPathLength);
}
//---------------------------------------------------------------------------
int __fastcall TDroppedFiles::GetCount()
{
	AssureUnfinished();
	return ::DragQueryFile(Handle, 0xFFFFFFFF, NULL, 0);
}
//---------------------------------------------------------------------------
TPoint __fastcall TDroppedFiles::GetPoint()
{
	AssureUnfinished();
	TPoint LPoint(0, 0);
	::DragQueryPoint(Handle, &LPoint);
	return LPoint;
}
//---------------------------------------------------------------------------
__fastcall TDroppedFiles::TDroppedFiles(const TMessage& AMessage) :
	TObject(),
	FHandle(reinterpret_cast<HDROP>(AMessage.WParam))
{
	// Do nothing
}
//---------------------------------------------------------------------------
__fastcall TDroppedFiles::~TDroppedFiles()
{
	// Do nothing
}
//---------------------------------------------------------------------------
void __fastcall TDroppedFiles::Finish()
{
	if(IsFinished()) {
		return;
	}
	::DragFinish(Handle);
	FHandle = NULL;
}
//---------------------------------------------------------------------------
bool __fastcall TDroppedFiles::IsFinished()
{
	return Handle == NULL;
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
void __fastcall TFileDropWatcher::Initialize()
{
	try {
		FNextHandleMessage = FControl->WindowProc;
		FControl->WindowProc = HandleMessage;
	}
	catch(...) {
		Uninitialize();
		throw;
	}
}
//---------------------------------------------------------------------------
void __fastcall TFileDropWatcher::Uninitialize()
{
	if(FNextHandleMessage != NULL) {
		Control->WindowProc = FNextHandleMessage;
		FNextHandleMessage = NULL;
	}
}
//---------------------------------------------------------------------------
void __fastcall TFileDropWatcher::HandleMessage(TMessage& AMessage)
{
	if(AMessage.Msg == WM_DROPFILES && WMDropFiles(AMessage)) {
		return;
	}
	NextHandleMessage(AMessage);
}
//---------------------------------------------------------------------------
bool __fastcall TFileDropWatcher::WMDropFiles(TMessage& AMessage)
{
	TAutoPointer<TDroppedFiles> LFiles;
	try {
		LFiles = new TDroppedFiles(AMessage);
		if(!DoFileDrop(LFiles)) {
			return false;
		}
	}
	catch(...) {
		if(LFiles == NULL) {
			::DragFinish(reinterpret_cast<HDROP>(AMessage.WParam));
		}
		else {
			LFiles->Finish();
		}
		throw;
	}
	LFiles->Finish();
	return true;
}
//---------------------------------------------------------------------------
bool __fastcall TFileDropWatcher::GetEnabled()
{
	::SetLastError(NO_ERROR);
	LONG_PTR LStyle = ::GetWindowLongPtr(Control->Handle, GWL_EXSTYLE);
	if(::GetLastError() != NO_ERROR) {
		throw EFileDropError("The program could not get the property.");
	}
	return (LStyle & WS_EX_ACCEPTFILES) != 0;
}
//---------------------------------------------------------------------------
void __fastcall TFileDropWatcher::SetEnabled(bool AEnabled)
{
	if((AEnabled ? 1 : 0) == (Enabled ? 1 : 0)) {
		return;
	}
	::DragAcceptFiles(Control->Handle, AEnabled);
}
//---------------------------------------------------------------------------
bool __fastcall TFileDropWatcher::DoFileDrop(TDroppedFiles* AFiles)
{
	bool AHandled = false;
	if(OnFileDrop != NULL) {
		OnFileDrop(this, AFiles, AHandled);
	}
	return AHandled;
}
//---------------------------------------------------------------------------
__fastcall TFileDropWatcher::TFileDropWatcher(TWinControl* AControl) :
	TObject(),
	FControl(AControl),
	FNextHandleMessage(NULL),
	FOnFileDrop(NULL)
{
	Initialize();
}
//---------------------------------------------------------------------------
__fastcall TFileDropWatcher::~TFileDropWatcher()
{
	Uninitialize();
}
//---------------------------------------------------------------------------
} // End namespace Moa_shelllibrary
//---------------------------------------------------------------------------

