//---------------------------------------------------------------------------
// Version: 1.3.0
// Date: 2012-07-12 UTC+09:00
// Author: Moochos
//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop
#include "Moa_AutoPointers.h"
#include "Moa_Strings.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
//---------------------------------------------------------------------------
namespace Moa_strings {
//---------------------------------------------------------------------------
AnsiString __fastcall TAnsiStringUtilities::Quote(const AnsiString& AString, char AQuoteCharacter)
{
	if(::IsDBCSLeadByte(AQuoteCharacter) || AQuoteCharacter == '\0') {
		throw EStringError("The program cannot quote the string.");
	}
	return Sysutils::AnsiQuotedStr(AString, AQuoteCharacter);
}
//---------------------------------------------------------------------------
AnsiString __fastcall TAnsiStringUtilities::Dequote(const AnsiString& AString, char AQuoteCharacter)
{
	if(::IsDBCSLeadByte(AQuoteCharacter) || AQuoteCharacter == '\0' ||
		AString.Length() == 0 || AString[1] != AQuoteCharacter) {
		throw EStringError("The program cannot dequote the string.");
	}
	char* LPosition = AString.c_str();
	return Sysutils::AnsiExtractQuotedStr(LPosition, AQuoteCharacter);
}
//---------------------------------------------------------------------------
AnsiString __fastcall TAnsiStringUtilities::DoubleEscape(const AnsiString& AString, char AEscapeCharacter)
{
	return Replace(
		AString,
		AEscapeCharacter,
		AnsiString::StringOfChar(AEscapeCharacter, 2),
		true,
		true);
}
//---------------------------------------------------------------------------
AnsiString __fastcall TAnsiStringUtilities::DoubleUnescape(const AnsiString& AString, char AEscapeCharacter)
{
	return Replace(
		AString,
		AnsiString::StringOfChar(AEscapeCharacter, 2),
		AEscapeCharacter,
		true,
		true);
}
//---------------------------------------------------------------------------
AnsiString __fastcall TAnsiStringUtilities::Replace(const AnsiString& AString, const AnsiString& AOldString, const AnsiString& ANewString, bool ACaseSensitive, bool AReplaceAll)
{
	Sysutils::TReplaceFlags LFlags;
	if(!ACaseSensitive) {
		LFlags << Sysutils::rfIgnoreCase;
	}
	if(AReplaceAll) {
		LFlags << Sysutils::rfReplaceAll;
	}
	return Sysutils::StringReplace(AString, AOldString, ANewString, LFlags);
}
//---------------------------------------------------------------------------
AnsiString __fastcall TAnsiStringUtilities::CommaTextToText(const AnsiString& ACommaText)
{
	return DelimitedTextToText(ACommaText, ',', '\"');
}
//---------------------------------------------------------------------------
AnsiString __fastcall TAnsiStringUtilities::TextToCommaText(const AnsiString& AText)
{
	return TextToDelimitedText(AText, ',', '\"');
}
//---------------------------------------------------------------------------
AnsiString __fastcall TAnsiStringUtilities::DelimitedTextToText(const AnsiString& ADelimitedText, char ADelimiter, char AQuoteCharacter)
{
	TAutoPointer<TStrings> LParser = new TStringList();
	LParser->Delimiter = ADelimiter;
	LParser->QuoteChar = AQuoteCharacter;
	LParser->DelimitedText = ADelimitedText;
	return LParser->Text;
}
//---------------------------------------------------------------------------
AnsiString __fastcall TAnsiStringUtilities::TextToDelimitedText(const AnsiString& AText, char ADelimiter, char AQuoteCharacter)
{
	TAutoPointer<TStrings> LParser = new TStringList();
	LParser->Delimiter = ADelimiter;
	LParser->QuoteChar = AQuoteCharacter;
	LParser->Text = AText;
	return LParser->DelimitedText;
}
//---------------------------------------------------------------------------
DynamicArray<AnsiString> __fastcall TAnsiStringUtilities::CommaTextToArray(const AnsiString& ACommaText)
{
	return DelimitedTextToArray(ACommaText, ',', '\"');
}
//---------------------------------------------------------------------------
AnsiString __fastcall TAnsiStringUtilities::ArrayToCommaText(const DynamicArray<AnsiString>& AArray)
{
	return ArrayToDelimitedText(AArray, ',', '\"');
}
//---------------------------------------------------------------------------
DynamicArray<AnsiString> __fastcall TAnsiStringUtilities::DelimitedTextToArray(const AnsiString& ADelimitedText, char ADelimiter, char AQuoteCharacter)
{
	TAutoPointer<TStrings> LParser = new TStringList();
	LParser->Delimiter = ADelimiter;
	LParser->QuoteChar = AQuoteCharacter;
	LParser->DelimitedText = ADelimitedText;
	DynamicArray<AnsiString> LArray;
	LArray.Length = LParser->Count;
	for(int LI = 0; LI < LArray.Length; ++LI) {
		LArray[LI] = LParser->Strings[LI];
	}
	return LArray;
}
//---------------------------------------------------------------------------
AnsiString __fastcall TAnsiStringUtilities::ArrayToDelimitedText(const DynamicArray<AnsiString>& AArray, char ADelimiter, char AQuoteCharacter)
{
	TAutoPointer<TStrings> LParser = new TStringList();
	LParser->Delimiter = ADelimiter;
	LParser->QuoteChar = AQuoteCharacter;
	LParser->Capacity = AArray.Length;
	for(int LI = 0; LI < AArray.Length; ++LI) {
		LParser->Add(AArray[LI]);
	}
	return LParser->DelimitedText;
}
//---------------------------------------------------------------------------
AnsiString __fastcall TAnsiStringUtilities::FormatInteger(int AInteger, int AGroupDigits, const AnsiString& LGroupSeparator)
{
	return TAnsiStringUtilities::FormatInteger(static_cast<__int64>(AInteger), AGroupDigits, LGroupSeparator);
}
//---------------------------------------------------------------------------
AnsiString __fastcall TAnsiStringUtilities::FormatInteger(__int64 AInteger, int AGroupDigits, const AnsiString& LGroupSeparator)
{
	AnsiString LString = AInteger;
	int LDigits = AInteger < 0 ? (LString.Length() - 1) : LString.Length();
	int LIndex = LDigits + 1;
	while((LIndex -= AGroupDigits) > 1) {
		LString.Insert(LGroupSeparator, LIndex);
	}
	return LString;
}
//---------------------------------------------------------------------------
AnsiString __fastcall TAnsiStringUtilities::FormatInteger(unsigned int AInteger, int AGroupDigits, const AnsiString& LGroupSeparator)
{
	return TAnsiStringUtilities::FormatInteger(static_cast<unsigned __int64>(AInteger), AGroupDigits, LGroupSeparator);
}
//---------------------------------------------------------------------------
AnsiString __fastcall TAnsiStringUtilities::FormatInteger(unsigned __int64 AInteger, int AGroupDigits, const AnsiString& LGroupSeparator)
{
	AnsiString LString = AInteger;
	int LDigits = AInteger < 0 ? (LString.Length() - 1) : LString.Length();
	int LIndex = LDigits + 1;
	while((LIndex -= AGroupDigits) > 1) {
		LString.Insert(LGroupSeparator, LIndex);
	}
	return LString;
}
//---------------------------------------------------------------------------
} // End namespace Moa_strings
//---------------------------------------------------------------------------

